#########################################################################
# calcDaysOpen.pl
# 
# Description: 
#    This transformation determines the number of days a defect has been
#    opened.
#
#    An "open" defect is defined to be those defects that with state of
#    'Submitted', 'Assigned', 'Open', or 'Resolved'.  If your definition is 
#    different, then you must change the query.
#########################################################################
use PjCLib;
use Win32::OLE;
require CQPerlExt;


################
# main
################

# global variables
$currentDate; 
$currentTime;
getCurrentDateTime();

# Get the ProjectConsole CQ session variable.
$session = logon();

myMain();
exit;

sub myMain {

    # You must always call the initializePjCLibrary() first before
    # you can use the PjCLib.
    PjCLib->initializePjCLibrary($session, PjCLib->BOOLEAN_TRUE);


    ###################################################
    #  Your transformation code goes here
    print "Calculate the number of days change requests have been open since the day they were submitted.\n";
    print "Today's date is $currentDate.\n";
    print "Please wait...\n";

    # First, get the physical names of the PjC tables we wish to query...
    # NOTE: If your table and field names are different, make the appropriate 
    # changes here!  These names are case sensitive!
    $DAYSOPEN_TABLE = PjCLib->getPhysicalTableDbName("DaysOpen_Dim");
    $STATE_TABLE = PjCLib->getPhysicalTableDbName("state_dim");
    $SUBMDATE_TABLE = PjCLib->getPhysicalTableDbName("SubmitDate");
    $DEFECT_TABLE = PjCLib->getPhysicalTableDbName("defect_measure");
    $COLLECTIONDATE_TABLE = PjCLib->getPhysicalTableDbName("collectiondate");

    $DAYSOPEN_FK = PjCLib->getPhysicalFieldDbName("defect_measure", "DaysOpen_fk");
    $COLLECTIONDATE_FK = PjCLib->getPhysicalFieldDbName("defect_measure", "collectiondate_fk");
    $SUBMITDATE_FK = PjCLib->getPhysicalFieldDbName("defect_measure", "SubmitDate_fk");
    $STATE_FK = PjCLib->getPhysicalFieldDbName("defect_measure", "state_fk");

    $DAYSOPEN_FIELD = PjCLib->getPhysicalFieldDbName("DaysOpen_Dim", "Days Open");
    $SUBMITDATE_NAME_FIELD = PjCLib->getPhysicalFieldDbName("SubmitDate", "SubmitDate_name");
    $SUBMITDATE_DAYNUM_FIELD = PjCLib->getPhysicalFieldDbName("SubmitDate", "SubmitDate_daynum");
    $COLLECTIONDATE_NAME_FIELD = PjCLib->getPhysicalFieldDbName("collectiondate", "collectiondate_name");
    $STATE_FIELD = PjCLib->getPhysicalFieldDbName("state_dim", "state");


    # Get submitdate_daynum for today's date ($currentDate).  We will subtract the
    # defect's submitdate_daynum to get the number of days open.
    $queryStr = "SELECT $SUBMITDATE_DAYNUM_FIELD FROM $SUBMDATE_TABLE  " .
                "WHERE $SUBMITDATE_NAME_FIELD='$currentDate'";
    #print "Excute query:\n";
    #print "$queryStr...\n";

    my $resultSetObj = $session->BuildSQLQuery($queryStr);
    $resultSetObj->Execute();
    if ($resultSetObj->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
       $currentDayNum = $resultSetObj->GetColumnValue(1);
       #print "currentDayNum = $currentDayNum\n";
    } 
    else {
       print "ERROR: Could not get currentDayNum!\n";
       return;
    }

    #
    # Get dbid and submitdate.submitdate_daynum for each open defect collected today.
    # 
    # Sample query that will be executed looks like the following:
    #   SELECT defect.dbid,submitdate.submitdate_daynum 
    #   FROM defect,submitdate
    #   WHERE (defect.collectiondate_fk=(SELECT collectiondate.dbid FROM collectiondate WHERE collectiondate.collectiondate_name='2002-03-19') and
    #   defect.submitdate_fk=submitdate.dbid and 
    #   defect.state_fk in (select dbid from state_dim where state in ('Opened','Submitted','Assigned','Resolved'))

    $queryStr = "SELECT $DEFECT_TABLE.dbid,$SUBMDATE_TABLE.$SUBMITDATE_DAYNUM_FIELD " .
                    "FROM $DEFECT_TABLE,$SUBMDATE_TABLE " .
                    "WHERE ($DEFECT_TABLE.$COLLECTIONDATE_FK= " .
                               " (SELECT $COLLECTIONDATE_TABLE.dbid " .
                               "  FROM $COLLECTIONDATE_TABLE " .
                               "  WHERE $COLLECTIONDATE_TABLE.$COLLECTIONDATE_NAME_FIELD='$currentDate') and " .
                               "    $DEFECT_TABLE.$SUBMITDATE_FK=$SUBMDATE_TABLE.dbid and " .
                               "    $DEFECT_TABLE.$STATE_FK in (SELECT dbid FROM $STATE_TABLE " .
			       "      WHERE $STATE_FIELD in ('Opened','Submitted','Assigned','Resolved')))";

    #print "Executing query:\n";
    #print "$queryStr\n\n";
    my $resultSetObj = $session->BuildSQLQuery($queryStr);
    $resultSetObj->Execute();
    $count = 0;
    while ($resultSetObj->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
       $dbid = $resultSetObj->GetColumnValue(1);
       $submitdate_daynum = $resultSetObj->GetColumnValue(2);

       # The difference between the submitdate's daynum and the current date's daynum is
       # is the number of days open!
       $numDaysOpen = $currentDayNum - $submitdate_daynum; 

       updateDaysOpen($dbid, $numDaysOpen);
       $count++;
    }

    print "$count Change Request measures were updated.\n";
    #
    ###########################################################
} # end subroutine myMain()


sub updateDaysOpen()
{
    my ($dbid, $daysOpen,) = @_;

    # $daysOpenStr is prefixed by a number so that they show up 
    # sorted on the Dashboard distribution chart.
    if ($daysOpen < 15) {
        $daysOpenStr = "1: <15 days";
    } elsif (($daysOpen > 15) && ($daysOpen <= 30)) {
        $daysOpenStr = "2: 16-30 days";
    } elsif (($daysOpen > 30) && ($daysOpen <= 45)) {
        $daysOpenStr = "3: 31-45 days";
    } elsif (($daysOpened > 46) && ($daysOpen <= 60)) {
        $daysOpenStr = "4: 46-60 days";
    } elsif (($daysOpen > 61) && ($daysOpen <= 90)) {
        $daysOpenStr = "5: 61-90 days";
    } elsif (($daysOpen > 91) && ($daysOpen <= 180)) {
        $daysOpenStr = "6: 91-180 days";
    } elsif (($daysOpen > 181) && ($daysOpen <= 360)) {
        $daysOpenStr = "7: 181-360 days";
    } elsif($daysOpen > 360) {
        $daysOpenStr = "8: 360+ days";
    }  

    # Make sure the $daysOpenStr value is defined  
    # DaysOpen dimension table.  If it isn't, the
    # value needs to be inserted. Otherwise, you cannot
    # create a reference to it.
    $queryStr = "select dbid from $DAYSOPEN_TABLE " .
                "where $DAYSOPEN_FIELD = '$daysOpenStr'";
 
    my $resultSet = $session->BuildSQLQuery($queryStr);
    $resultSet->Execute();
    if ($resultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
      # If the value is already defined in the table, then
      # no need to do anything.
    } else {
       # do an INSERT
       $entity = $session->BuildEntity($DAYSOPEN_TABLE);
       $entity->SetFieldValue($DAYSOPEN_FIELD,$daysOpenStr);
       $retVal = $entity->Validate();
       $entity->Commit();
    }


    my $entity = $session->GetEntityByDbId("defect", $dbid);
    $session->EditEntity($entity,"modify");
    $entity->SetFieldValue($DAYSOPEN_FK, $daysOpenStr);
    $retVal = $entity->Validate();
    $entity->Commit();

 } # end subroutine updateDaysOpen()



sub logon {

    #
    # Use the internal DashboardSystem login to get access to the Dashboard database
    #
    my $session = Win32::OLE->new('CLEARQUEST.SESSION');
    my $session = CQPerlExt::CQSession_Build();  
    $session->UserLogon("dashboardsystem", "dashboardsystem", "tstd1", "Dashboard");
    return $session;
}

# get date and set the global variables $currentDate and $currentTime
sub getCurrentDateTime {
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);

    $mon = $mon + 1;
    $year = $year + 1900;

    if ($mday<10) {$mday="0$mday"; }  # add leading 0
    if ($mon<10) {$mon="0$mon"; }     # add leading 0

    # set the global variable $currentDate
    $currentDate="$year-$mon-$mday";

    # set the global variable $currentTime
    $currentTime = $hour . ":" . $min . ":" . $sec;

}

